/*
 * Copyright (C) Yao Jun 2011
 * <nter@tom.com> 
 * <http://CortOS.sourceforge.net/>
 *
 * SortOS is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SortOS is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "cortos.h"
#include "cortos_common.h"

#if (CORT_DAI_EN)

static t_sem 	 CORT_DAI_Sem;
static t_device *CORT_DAI_head = m_null;

/*
**
*/
void CORT_DAI_Init(void)
{
	CORT_Sem_Create(&CORT_DAI_Sem, 1);
}

/*
**
*/
t_s8 CORT_DAI_RegDev(t_device * pDev)
{
#if (CORT_CHECK_PARAM)
	if (pDev == m_null)
	{
		return CORT_ERR_INVALID_HANDLE;
	}
	if (pDev->dev_type > CORT_DAIT_I2C)
	{
		return CORT_ERROR;
	}
	if (pDev->open == m_null)
	{
		return CORT_ERROR;
	}
#endif

	if (CORT_OK != CORT_Sem_Obtain(&CORT_DAI_Sem, -1))
		return CORT_ERROR;

#if (CORT_DAI_READ_PROTECT_EN)
	CORT_Sem_Create(&pDev->sem_read, 1);
#endif
#if (CORT_DAI_WRITE_PROTECT_EN)
	CORT_Sem_Create(&pDev->sem_write, 1);
#endif
#if (CORT_DAI_IOCTRL_PROTECT_EN)
	CORT_Sem_Create(&pDev->sem_ioctrl, 1);
#endif

	pDev->next = CORT_DAI_head;
	CORT_DAI_head = pDev;
	
	CORT_Sem_Release(&CORT_DAI_Sem);
	return CORT_OK;
}

/*
**
*/
t_device *CORT_DAI_Open(t_u8 nDevType, t_u8 nDevIdx, t_s32 nTimOutTick)
{
	t_device *pDev;
#if (CORT_CHECK_PARAM)
	if (nDevType > CORT_DAIT_I2C)
	{
		return m_null;
	}
#endif
	if (CORT_OK != CORT_Sem_Obtain(&CORT_DAI_Sem, nTimOutTick))
		return m_null;

	pDev = CORT_DAI_head;
	while(pDev)
	{
		if (pDev->dev_type == nDevType &&
			pDev->dev_idx  == nDevIdx)
		{
			break;
		}
		pDev = pDev->next;
	}
	CORT_Sem_Release(&CORT_DAI_Sem);

	if (pDev)
	{
		if (CORT_OK != pDev->open())
		{
			return m_null;
		}
	}	
	return pDev;
}

/*
**
*/
#if (CORT_DAI_CLOSE)
t_s8 CORT_DAI_Close(t_device * pDev, t_s32 nTimOutTick)
{
	t_s8 ret;
#if (CORT_CHECK_PARAM)
	if (pDev == m_null)
	{
		return CORT_ERR_INVALID_HANDLE;
	}
#endif
	if (pDev->close == m_null)
	{
		return CORT_OK;
	}

	ret = pDev->close();
	
	return ret;
}
#endif
/*
**
*/
#if (CORT_DAI_READ)
t_s32 CORT_DAI_Read(t_device * pDev, t_u8* pBuf, t_s32 nLen, t_s32 nTimOutTick)
{
	t_s8 ret;
#if (CORT_CHECK_PARAM)
	if (pDev == m_null)
	{
		return -1;
	}
	if (pBuf == m_null || nLen <= 0)
	{
		return 0;
	}
#endif

	if (pDev->read == m_null)
	{
		return 0;
	}

#if (CORT_DAI_READ_PROTECT_EN)
	switch(CORT_Sem_Obtain(&pDev->sem_read, nTimOutTick))
	{
	case CORT_OK:
		break;
	case CORT_ERR_TIMEOUT:
		return 0;
	default:
		return -1;
	}
#endif

	ret = pDev->read(pBuf, nLen, nTimOutTick);
	
#if (CORT_DAI_READ_PROTECT_EN)
	CORT_Sem_Release(&pDev->sem_write);
#endif

	return ret;
}
#endif
/*
**
*/
#if (CORT_DAI_WRITE)
t_s32 CORT_DAI_Write(t_device * pDev, t_u8* pBuf, t_s32 nLen, t_s32 nTimOutTick)
{
	t_s8 ret;
#if (CORT_CHECK_PARAM)
	if (pDev == m_null)
	{
		return -1;
	}
	if (pBuf == m_null || nLen <= 0)
	{
		return 0;
	}
#endif


	if (pDev->write == m_null)
	{
		return 0;
	}

#if (CORT_DAI_WRITE_PROTECT_EN)
	switch(CORT_Sem_Obtain(&pDev->sem_write, nTimOutTick))
	{
	case CORT_OK:
		break;
	case CORT_ERR_TIMEOUT:
		return 0;
	default:
		return -1;
	}
#endif	

	ret = pDev->write(pBuf, nLen, nTimOutTick);

#if (CORT_DAI_WRITE_PROTECT_EN)
	CORT_Sem_Release(&pDev->sem_write);
#endif

	return ret;
}
#endif
/*
**
*/
#if (CORT_DAI_IOCTRL)
t_s8 CORT_DAI_IOCtrl(t_device * pDev, t_u8 nParam, t_handle hParam, t_s32 nTimOutTick)
{
	t_s8 ret;
#if (CORT_CHECK_PARAM)
	if (pDev == m_null)
	{
		return CORT_ERR_INVALID_HANDLE;
	}
#endif

	if (pDev->ioctrl == m_null)
	{
		return CORT_ERROR;
	}

#if (CORT_DAI_IOCTRL_PROTECT_EN)
	switch(CORT_Sem_Obtain(&pDev->sem_ioctrl, nTimOutTick))
	{
	case CORT_OK:
		break;
	case CORT_ERR_TIMEOUT:
		return 0;
	default:
		return -1;
	}
#endif

	ret = pDev->ioctrl(nParam, hParam, nTimOutTick);

#if (CORT_DAI_IOCTRL_PROTECT_EN)
	CORT_Sem_Release(&pDev->sem_ioctrl);
#endif

	return ret;
}
#endif

/*
**
*/
#if (CORT_DAI_STANDBY)
t_s8 CORT_DAI_Standby(t_device * pDev, t_s32 nTimOutTick)
{
	t_s8 ret;
	t_bool bOne = m_true;
	
	if (CORT_OK != (ret = CORT_Sem_Obtain(&CORT_DAI_Sem, nTimOutTick)))
	{
		return ret;
	}
	if (pDev)
	{
		bOne = m_false;
		pDev = CORT_DAI_head;
	}
	while(pDev)
	{
		if (pDev->standby == m_null)
		{
			pDev = pDev->next;
			continue;
		}
		if (CORT_OK != (ret = pDev->standby()))
		{
			CORT_Sem_Release(&CORT_DAI_Sem);
			return ret;
		}
		if (bOne)
		{
			break;
		}
		pDev = pDev->next;
	}
	CORT_Sem_Release(&CORT_DAI_Sem);
	return CORT_OK;
}
#endif

/*
**
*/
#if (CORT_DAI_STANDBY)
t_s8 CORT_DAI_Resume (t_device * pDev, t_s32 nTimOutTick)
{
	t_s8 ret;
	t_bool bOne = m_true;

	if (CORT_OK != (ret = CORT_Sem_Obtain(&CORT_DAI_Sem, nTimOutTick)))
	{
		return ret;
	}
	if (pDev)
	{
		bOne = m_false;
		pDev = CORT_DAI_head;
	}
	while(pDev)
	{
		if (pDev->resume == m_null)
		{
			pDev = pDev->next;
			continue;
		}
		if (CORT_OK != (ret = pDev->resume()))
		{
			CORT_Sem_Release(&CORT_DAI_Sem);
			return ret;
		}
		if (bOne)
		{
			break;
		}
		pDev = pDev->next;
	}
	CORT_Sem_Release(&CORT_DAI_Sem);
	return CORT_OK;
}
#endif

/*
**
*/
#if (CORT_DAI_STANDBY)
t_bool CORT_DAI_IsAllowStandby(t_device * pDev, t_s32 nTimOutTick)
{
	t_bool bOne = m_true;
	
	if (CORT_OK != CORT_Sem_Obtain(&CORT_DAI_Sem, nTimOutTick))
	{
		return m_false;
	}
	if (pDev)
	{
		bOne = m_false;
		pDev = CORT_DAI_head;
	}
	while(pDev)
	{
		if (pDev->allow_standby == m_null)
		{
			pDev = pDev->next;
			continue;
		}
		if (!pDev->allow_standby())
		{
			CORT_Sem_Release(&CORT_DAI_Sem);
			return m_false;
		}
		if (bOne)
		{
			break;
		}
		pDev = pDev->next;
	}
	CORT_Sem_Release(&CORT_DAI_Sem);
	return m_true;
}
#endif

#endif

