/*
 * Copyright (C) Yao Jun 2011
 * <nter@tom.com> 
 * <http://CortOS.sourceforge.net/>
 *
 * SortOS is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SortOS is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>. 
 */


/*
** Create Pipe
** Allowed From: Task Timer Interrupt
*/
#if (CORT_PIPE_EN)
t_s8  CORT_Pipe_Create(t_pipe *ppipe, t_u8 *pbuf, t_u16 num)
{
#if (CORT_CHECK_PARAM)
	if(ppipe == m_null)
	{
		return CORT_ERR_INVALID_HANDLE;
	}
	if (pbuf == m_null || num == 0)
	{
		return CORT_ERR_INVALID_PARAM;
	}
#endif
	CORT_EnterCritical();
#if (CORT_CHECK_ALIVE)
	if (ppipe->pe_alive_flag == CORT_ALIVE_FLAG_PIPE)
	{
		CORT_ExitCritical();
		return CORT_ERR_INVALID_HANDLE;
	}
	ppipe->pe_alive_flag = CORT_ALIVE_FLAG_PIPE;
#endif
	ppipe->pe_num = num;
	ppipe->pe_rear = 0;
	ppipe->pe_front = 0;
	ppipe->pe_buffer = pbuf;
	ppipe->pe_wtask = m_null;
	CORT_SemInit(&ppipe->pe_read,    0);
	CORT_SemInit(&ppipe->pe_write, num);

	CORT_ExitCritical();
	
	return CORT_OK;
}
#endif

/*
** Delete Pipe
** Allowed From: 
*/
#if (CORT_PIPE_EN && CORT_PIPE_DEL_EN)
t_s8  CORT_Pipe_Delete(t_pipe *ppipe)
{	
#if (CORT_CHECK_PARAM)
	if (ppipe == m_null)
	{
		return CORT_ERR_INVALID_HANDLE; 
	}
#endif
	CORT_EnterCritical();
#if (CORT_CHECK_ALIVE)
	if (ppipe->pe_alive_flag != CORT_ALIVE_FLAG_PIPE)
	{
		CORT_ExitCritical();
		return CORT_ERR_INVALID_HANDLE;
	}
	ppipe->pe_alive_flag = 0;
#endif

	CORT_DeleteSemaphore(&ppipe->pe_write, m_false);
	CORT_DeleteSemaphore(&ppipe->pe_read,  m_true);
	
	CORT_ExitCritical();
	return CORT_OK;
}
#endif

/*
** Write Pipe
** Allowed From: Task Timer Interrupt
*/
#if (CORT_PIPE_EN)
t_s8  CORT_Pipe_Write(t_pipe *ppipe, t_u8 * pbuf, t_s32 *pnum, t_s32 tick)
{
	t_s8 ret;
	t_u8 *pdata;
	t_s32 rear,max,num,wlen;
	t_sem * psem_w, *psem_r;

#if (CORT_CHECK_PARAM)
	if (ppipe == m_null)
	{
		return CORT_ERR_INVALID_HANDLE;
	}
	if (pbuf == m_null || pnum == m_null)
	{
		return CORT_ERR_INVALID_PARAM;
	}
#endif
	wlen = *pnum;
	*pnum = 0;
	if (wlen == 0)
	{
		return CORT_OK;
	}
	if (wlen < 0)
	{
		return CORT_ERR_INVALID_PARAM;
	}
	CORT_EnterCritical();
#if (CORT_CHECK_ALIVE)
	if (ppipe->pe_alive_flag != CORT_ALIVE_FLAG_PIPE)
	{
		CORT_ExitCritical();
		return CORT_ERR_INVALID_HANDLE;
	}
#endif

	psem_w = &ppipe->pe_write;
	psem_r = &ppipe->pe_read;
	do{
		while(!psem_w->sm_cnt)
		{
			ppipe->pe_wtask = (*pnum) ? CORT_CurTcb : m_null;
			
			ret = CORT_TaskToWait(psem_w, &tick, 0, 0);

			ppipe->pe_wtask = m_null;
			
			if (ret != CORT_OK)
			{
				CORT_ExitCritical();
				return ret;
			}
		}

		num = CORT_MIN(psem_w->sm_cnt, wlen);
		*pnum          += num;
		wlen           -= num;
		psem_w->sm_cnt -= num;
		psem_r->sm_cnt += num;
		max   = ppipe->pe_num;
		rear  = ppipe->pe_rear;
		pdata = ppipe->pe_buffer;
		while(num--)
		{
			pdata[rear] = *pbuf++;
			rear = (rear+1) % max;
		}
		ppipe->pe_rear = rear;
	}while(wlen);
	
	/* Check read task */
	CORT_ResumeTask(psem_r, m_null, m_true, CORT_OK);
	
	CORT_ExitCritical();
	return CORT_OK;
}
#endif

/*
** Read Pipe
** Allowed From: Task Timer Interrupt
*/
#if (CORT_PIPE_EN)
t_s8  CORT_Pipe_Read(t_pipe *ppipe, t_u8 * pbuf, t_s32 *pnum, t_s32 tick)
{
	t_s8 ret;
	t_u8 *pdata;
	t_s32 front, max, num, rlen;
	t_sem * psem_w, *psem_r;
	
#if (CORT_CHECK_PARAM)
	if(ppipe == m_null)
	{
		return CORT_ERR_INVALID_HANDLE;
	}
	if (pbuf == m_null || pnum == m_null)
	{
		return CORT_ERR_INVALID_PARAM;
	}
#endif

	rlen  = *pnum;
	*pnum = 0;
	if (rlen == 0)
	{
		return CORT_OK;
	}
	if (rlen < 0)
	{
		return CORT_ERR_INVALID_PARAM;
	}
	
	CORT_EnterCritical();
#if (CORT_CHECK_ALIVE)
	if (ppipe->pe_alive_flag != CORT_ALIVE_FLAG_PIPE)
	{
		CORT_ExitCritical();
		return CORT_ERR_INVALID_HANDLE;
	}
#endif

	psem_w = &ppipe->pe_write;
	psem_r = &ppipe->pe_read;

	do{
		while (!psem_r->sm_cnt)
		{			
			ret = CORT_TaskToWait(psem_r, &tick, 0, 0);
			if (ret != CORT_OK)
			{
				CORT_ExitCritical();
				return ret;
			}
		}

		num = CORT_MIN(psem_r->sm_cnt, rlen);
		*pnum          += num;
		rlen           -= num;
		psem_r->sm_cnt -= num;
		psem_w->sm_cnt += num;
		max   = ppipe->pe_num;
		front = ppipe->pe_front;
		pdata = ppipe->pe_buffer;
		while(num--)
		{
			*pbuf++ = pdata[front];
			front = (front+1) % max;
		}
		ppipe->pe_front = front;

	}while(0);
	
	/* Check Write task*/
	CORT_ResumeTask(psem_w, ppipe->pe_wtask, m_true, CORT_OK);
		
	CORT_ExitCritical();
	return CORT_OK;
}
#endif

/*
** Reset Pipe
** Allowed From: Task Timer Interrupt
*/
#if (CORT_PIPE_EN)
t_s8  CORT_Pipe_Reset(t_pipe *ppipe)
{
#if (CORT_CHECK_PARAM)
	if(ppipe == m_null)
	{
		return CORT_ERR_INVALID_HANDLE;
	}
#endif
	CORT_EnterCritical();
#if (CORT_CHECK_ALIVE)
	if (ppipe->pe_alive_flag != CORT_ALIVE_FLAG_PIPE)
	{
		CORT_ExitCritical();
		return CORT_ERR_INVALID_HANDLE;
	}
#endif

	ppipe->pe_rear = 0;
	ppipe->pe_front = 0;
	ppipe->pe_read.sm_cnt = 0;
	ppipe->pe_write.sm_cnt = ppipe->pe_num;

	CORT_ExitCritical();
	return CORT_OK;
}
#endif


