/*
 * Copyright (C) Yao Jun 2011
 * <nter@tom.com> 
 * <http://CortOS.sourceforge.net/>
 *
 * SortOS is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SortOS is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


/*
** Create Queue
** Allowed From: Task Timer Interrupt
*/
#if (CORT_QUEUE_EN)
t_s8  CORT_Queue_Create(t_queue *pqueue, t_void ** ppbuf, t_u32 num)
{
#if (CORT_CHECK_PARAM)
	if (pqueue == m_null)
	{
		return CORT_ERR_INVALID_HANDLE;
	}
	if (ppbuf == m_null || num == 0)
	{
		return CORT_ERR_INVALID_PARAM;
	}
#endif
	CORT_EnterCritical();
#if (CORT_CHECK_ALIVE)
	if (pqueue->qu_alive_flag == CORT_ALIVE_FLAG_QUEUE)
	{
		CORT_ExitCritical();
		return CORT_ERR_INVALID_HANDLE;
	}
	pqueue->qu_alive_flag = CORT_ALIVE_FLAG_QUEUE;
#endif
	pqueue->qu_num = num;
	pqueue->qu_rear = 0;
	pqueue->qu_front = 0;
	pqueue->qu_buffer = ppbuf;
	CORT_SemInit(&pqueue->qu_read,    0);
	CORT_SemInit(&pqueue->qu_write, num);

	CORT_ExitCritical();
	
	return CORT_OK;
}
#endif

/*
** Delete Queue
** Allowed From: 
*/
#if (CORT_QUEUE_EN && CORT_QUEUE_DEL_EN)
t_s8  CORT_Queue_Delete(t_queue *pqueue)
{
#if (CORT_CHECK_PARAM)
	if (pqueue == m_null)
	{
		return CORT_ERR_INVALID_HANDLE; 
	}
#endif
	CORT_EnterCritical();
#if (CORT_CHECK_ALIVE)
	if (pqueue->qu_alive_flag != CORT_ALIVE_FLAG_QUEUE)
	{
		CORT_ExitCritical();
		return CORT_ERR_INVALID_HANDLE;
	}
	pqueue->qu_alive_flag = 0;
#endif

	CORT_DeleteSemaphore(&pqueue->qu_write, m_false);
	CORT_DeleteSemaphore(&pqueue->qu_read,  m_true);
	
	CORT_ExitCritical();
	return CORT_OK;
}
#endif

/*
** Write Queue
** Allowed From: Task Timer Interrupt
*/
#if (CORT_QUEUE_EN)
t_s8  CORT_Queue_Write(t_queue *pqueue, t_void * pbuf, t_bool bRear, t_s32 tick)
{
	t_s8 ret;
	t_u16 rear,front;
	t_sem * psem_r, * psem_w;

#if (CORT_CHECK_PARAM)
	if (pqueue == m_null)
	{
		return CORT_ERR_INVALID_HANDLE;
	}
	if (pbuf == m_null)
	{
		return CORT_ERR_INVALID_PARAM;
	}
#endif
	CORT_EnterCritical();
#if (CORT_CHECK_ALIVE)
	if (pqueue->qu_alive_flag != CORT_ALIVE_FLAG_QUEUE)
	{
		CORT_ExitCritical();
		return CORT_ERR_INVALID_HANDLE;
	}
#endif

	psem_r = &pqueue->qu_read;
	psem_w = &pqueue->qu_write;
	
	while(!psem_w->sm_cnt)
	{
		ret = CORT_TaskToWait(psem_w, &tick, 0, 0);
		if (ret != CORT_OK)
		{
			CORT_ExitCritical();
			return ret;
		}
	}
	
	/* Write point to queue */
	if (bRear)
	{
		rear                    = pqueue->qu_rear;
		pqueue->qu_buffer[rear] = pbuf;
		pqueue->qu_rear         = (rear+1) % pqueue->qu_num;
	}
	else
	{
		if (pqueue->qu_front == 0)
		{
			front = pqueue->qu_front 
				  = pqueue->qu_num - 1;
		}
		else
		{
			front = pqueue->qu_front 
				  = pqueue->qu_front - 1;
		}
		
		pqueue->qu_buffer[front] = pbuf;
	}
	psem_w->sm_cnt--;
	psem_r->sm_cnt++;
	
	/* Check read task */
	CORT_ResumeTask(psem_r, m_null, m_true, CORT_OK);
	
	CORT_ExitCritical();
	return CORT_OK;
}
#endif

/*
** Read Queue
** Allowed From: Task Timer Interrupt
*/
#if (CORT_QUEUE_EN)
t_s8  CORT_Queue_Read(t_queue *pqueue, t_void ** ppbuf, t_bool bFront, t_s32 tick)
{
	t_s8 ret;
	t_u16 rear,front;
	t_sem * psem_r, * psem_w;
	
#if (CORT_CHECK_PARAM)
	if (pqueue == m_null)
	{
		return CORT_ERR_INVALID_HANDLE;
	}
	if (ppbuf == m_null)
	{
		return CORT_ERR_INVALID_PARAM;
	}
#endif
	CORT_EnterCritical();
#if (CORT_CHECK_ALIVE)
	if (pqueue->qu_alive_flag != CORT_ALIVE_FLAG_QUEUE)
	{
		CORT_ExitCritical();
		return CORT_ERR_INVALID_HANDLE;
	}
#endif

	psem_r = &pqueue->qu_read;
	psem_w = &pqueue->qu_write;
	
	while(!psem_r->sm_cnt)
	{
		ret = CORT_TaskToWait(psem_r, &tick, 0, 0);
		if (ret != CORT_OK)
		{
			CORT_ExitCritical();
			return ret;
		}
	}
	
	/* Read point from queue */
	if (bFront)
	{
		front            = pqueue->qu_front;
		*ppbuf           = pqueue->qu_buffer[front];
		pqueue->qu_front = (front+1) % pqueue->qu_num;
	}
	else
	{
		if (pqueue->qu_rear == 0)
		{
			rear = pqueue->qu_rear 
				 = pqueue->qu_num - 1;
		}
		else
		{
			rear = pqueue->qu_rear 
				 = pqueue->qu_rear - 1;
		}

		*ppbuf = pqueue->qu_buffer[rear];
	}
	psem_r->sm_cnt--;
	psem_w->sm_cnt++;
	
	/* Check Write task*/
	CORT_ResumeTask(psem_w, m_null, m_true, CORT_OK);
	
	CORT_ExitCritical();
	return CORT_OK;
}
#endif


