/*
 * Copyright (C) Yao Jun 2011
 * <nter@tom.com> 
 * <http://CortOS.sourceforge.net/>
 *
 * SortOS is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SortOS is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>. 
 */

#include "cortos.h"
#include "cortos_common.h"

/*
** Create Timer
** Allowed From: Task Timer Interrupt
*/
#if (CORT_TIMER_EN)
t_s8  CORT_Timer_Create(t_timer *ptimer, t_u8 flag, t_u32 inittick, t_u32 schedtick, t_timproc func)
{
#if (CORT_CHECK_PARAM)
	if (ptimer == m_null)
	{
		return CORT_ERR_INVALID_HANDLE;
	}
	if (func == m_null || schedtick == 0)
	{
		return CORT_ERR_INVALID_PARAM;
	}
#endif
	CORT_EnterCritical();

#if (CORT_CHECK_ALIVE)
	ptimer->tm_alive_flag = CORT_ALIVE_FLAG_TIMER;
#endif
	ptimer->tm_flag   = flag;
	ptimer->tm_remain = inittick;
	ptimer->tm_reload = schedtick;
	ptimer->tm_loopcnt= -1;
	ptimer->tm_func   = func;
	ptimer->tm_next   = CORT_TimerList;

#if CORT_TIMER_DEL_EN
	ptimer->tm_prev   = m_null;
	if (CORT_TimerList)
	{
		CORT_TimerList->tm_prev = ptimer;
	}
#endif	
	CORT_TimerList = ptimer;

	CORT_ExitCritical();

	return CORT_OK;
}
#endif 

/*
** Delete Timer
** Allowed From: Task
*/
#if (CORT_TIMER_EN && CORT_TIMER_DEL_EN)
t_s8  CORT_Timer_Delete(t_timer *ptimer)
{
	t_timer *ptemp;
	
#if (CORT_CHECK_PARAM)
	if (ptimer == m_null)
	{
		return CORT_ERR_INVALID_HANDLE;
	}
#endif

	CORT_EnterCritical();

#if (CORT_CHECK_ALIVE)
	if (ptimer->tm_alive_flag != CORT_ALIVE_FLAG_TIMER)
	{
		CORT_ExitCritical();
		return CORT_ERR_INVALID_HANDLE;
	}
#endif

	ptemp = ptimer->tm_prev;
	if (ptemp != m_null)
	{
		ptemp->tm_next = ptimer->tm_next;
	}
	else
	{
		CORT_TimerList = ptimer->tm_next;
	}
	ptemp = ptimer->tm_next;
	if (ptemp != m_null)
	{
		ptemp->tm_prev = ptimer->tm_prev;
	}
#if (CORT_CHECK_ALIVE)
	ptimer->tm_alive_flag = 0;
#endif	

	CORT_ExitCritical();
	
	return CORT_OK;
}
#endif 

/*
** Reset Timer to start
** Allowed From: Task Timer Interrupt
*/
#if (CORT_TIMER_EN && CORT_TIMER_RESET_EN)
t_s8  CORT_Timer_Reset(t_timer *ptimer)
{
#if (CORT_CHECK_PARAM)
	if (ptimer == m_null)
	{
		return CORT_ERR_INVALID_HANDLE;
	}
#endif 

	CORT_EnterCritical();
#if (CORT_CHECK_ALIVE)
	if (ptimer->tm_alive_flag != CORT_ALIVE_FLAG_TIMER)
	{
		CORT_ExitCritical();
		return CORT_ERR_INVALID_HANDLE;
	}
#endif
 
	ptimer->tm_remain = ptimer->tm_reload;

	CORT_ExitCritical();	
	return CORT_OK;
}
#endif 

/*
** Start Timer
** Allowed From: Task Timer Interrupt
*/
#if (CORT_TIMER_EN && CORT_TIMER_STARTSTOP_EN)
t_s8  CORT_Timer_Start(t_timer *ptimer, t_u32 inittick, t_s32 loopcnt)
{	
#if (CORT_CHECK_PARAM)
	if (ptimer == m_null)
	{
		return CORT_ERR_INVALID_HANDLE;
	}
#endif

	CORT_EnterCritical();
#if (CORT_CHECK_ALIVE)
	if (ptimer->tm_alive_flag != CORT_ALIVE_FLAG_TIMER)
	{
		CORT_ExitCritical();
		return CORT_ERR_INVALID_HANDLE;
	}
#endif

	if (inittick > ptimer->tm_remain)
	{
		ptimer->tm_remain = inittick;
	}
	ptimer->tm_loopcnt= loopcnt;

	CORT_ExitCritical();	
	return CORT_OK;
}
#endif 

/*
** Stop Timer
** Allowed From: Task Timer Interrupt
*/
#if (CORT_TIMER_EN && CORT_TIMER_STARTSTOP_EN)
t_s8  CORT_Timer_Stop(t_timer *ptimer)
{	
#if (CORT_CHECK_PARAM)
	if (ptimer == m_null)
	{
		return CORT_ERR_INVALID_HANDLE;
	}
#endif

	CORT_EnterCritical();
#if (CORT_CHECK_ALIVE)
	if (ptimer->tm_alive_flag != CORT_ALIVE_FLAG_TIMER)
	{
		CORT_ExitCritical();
		return CORT_ERR_INVALID_HANDLE;
	}
#endif

	ptimer->tm_remain = 0;	

	CORT_ExitCritical();
	return CORT_OK;
}
#endif 

/*
** Set Timer Parameter
** Allowed From: Task Timer Interrupt
*/
#if (CORT_TIMER_EN && CORT_TIMER_SET_EN)
t_s8 CORT_Timer_Set(t_timer *ptimer, t_u8 flag, t_u32 schedtick)
{	
#if (CORT_CHECK_PARAM)
	if (ptimer == m_null)
	{
		return CORT_ERR_INVALID_HANDLE;
	}
#endif

	CORT_EnterCritical();
#if (CORT_CHECK_ALIVE)
	if (ptimer->tm_alive_flag != CORT_ALIVE_FLAG_TIMER)
	{
		CORT_ExitCritical();
		return CORT_ERR_INVALID_HANDLE;
	}
#endif

	ptimer->tm_flag = flag;
	ptimer->tm_reload = schedtick;
	
	CORT_ExitCritical();
	return CORT_OK;
}
#endif 


