/*
 * Copyright (C) Yao Jun 2013
 * <nter@tom.com> 
 * <http://CortOS.sourceforge.net/>
 *
 * SortOS is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SortOS is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "config.h"

enum{
	eI2CEvt_OK			= 0x01,
	eI2CEvt_Error		= 0x02,
	eI2CEvt_MASK		= 0x03,
};

static t_bool          i2c_1_init = m_false;
static struct t_device i2c_1_dev = {0};

static t_sem     i2c_sem;
static t_flag    i2c_flag;
static t_i2c_opr i2c_oper;
static t_u8      i2c_reg_cnt;	
static t_u16     i2c_read_cnt;
static t_u16     i2c_write_cnt;


/*
**  Interrupt Service
*/
static void dev_i2c_1_irq(t_u16 idx)
{
	t_u8 offset;
	uint32_t u32Status;

	u32Status = I2C0->I2CSTATUS;
	if (I2C0->I2CTOC & I2C_I2CTOC_TIF_Msk)
	{
		/* Clear I2C0 Timeout Flag */
		I2C0->I2CTOC |= I2C_I2CTOC_TIF_Msk;                   
	}    
	else
	{
		if (u32Status == 0x08)                      /* START has been transmitted and prepare SLA+W */
	    {
	       I2C0->I2CDAT = i2c_oper.dev_addr;	    /* Write SLA+W to Register I2CDAT */
	        _I2C_SET_CONTROL_BITS(I2C0, I2C_I2CON_SI);
	    }
		else if (u32Status == 0x20)                 /* SLA+W has been transmitted and NACK has been received */
	    {
			_I2C_SET_CONTROL_BITS(I2C0, I2C_I2CON_STO_SI);
			CORT_Flag_Set(&i2c_flag, eI2CEvt_Error, CORT_OR);
	    }

		else if (u32Status == 0x18 ||               /* SLA+W has been transmitted and ACK has been received */
				 u32Status == 0x28)                 /* DATA has been transmitted and ACK has been received */
	    {
			if (i2c_reg_cnt < i2c_oper.subreg_len)
			{
				if (i2c_oper.subreg_bMSB)
					offset = i2c_oper.subreg_len - i2c_reg_cnt - 1;
				else
					offset = i2c_reg_cnt;
					
				 I2C0->I2CDAT = (i2c_oper.subreg_val >> (offset * 8)) & 0xff;
				 _I2C_SET_CONTROL_BITS(I2C0, I2C_I2CON_SI);
				
				i2c_reg_cnt++;
			}
			else if (i2c_write_cnt < i2c_oper.write_len)
			{
				 I2C0->I2CDAT = i2c_oper.write_buf[i2c_write_cnt++];
				 _I2C_SET_CONTROL_BITS(I2C0, I2C_I2CON_SI);
			}
			else if (i2c_oper.read_len)
			{
				if (i2c_oper.write_endstop)
				{
					 _I2C_SET_CONTROL_BITS(I2C0, I2C_I2CON_STA_STO_SI);
				}
				else
				{
					_I2C_SET_CONTROL_BITS(I2C0, I2C_I2CON_STA_SI);
				}
			}
			else
			{
				_I2C_SET_CONTROL_BITS(I2C0, I2C_I2CON_STO_SI);
				CORT_Flag_Set(&i2c_flag, eI2CEvt_OK, CORT_OR);
			}			
	    }
	    else if (u32Status == 0x10)                 /* Repeat START has been transmitted and prepare SLA+R */
	    {
	        I2C0->I2CDAT = i2c_oper.dev_addr | 0x01;   /* Write SLA+R to Register I2CDAT */
	        _I2C_SET_CONTROL_BITS(I2C0, I2C_I2CON_SI);
	    }
	    else if (u32Status == 0x40)                 /* SLA+R has been transmitted and ACK has been received */
	    {
			i2c_oper.read_buf[i2c_read_cnt++] = I2C0->I2CDAT;

			if (1 == (i2c_oper.read_len-i2c_read_cnt))
			{
				_I2C_SET_CONTROL_BITS(I2C0, I2C_I2CON_SI);
			} 
			else
			{
				_I2C_SET_CONTROL_BITS(I2C0, I2C_I2CON_SI_AA); 
			}
	    }       
	    else if (u32Status == 0x58)                 /* DATA has been received and NACK has been returned */
	    { 
			i2c_oper.read_buf[i2c_read_cnt++] = I2C0->I2CDAT;
			_I2C_SET_CONTROL_BITS(I2C0, I2C_I2CON_STO_SI);
			
			if (i2c_read_cnt >= i2c_oper.read_len)
			{
				CORT_Flag_Set(&i2c_flag, eI2CEvt_OK, CORT_OR);
			}
			else
			{
				CORT_Flag_Set(&i2c_flag, eI2CEvt_Error, CORT_OR);
			}
	    }
	    else
	    {
	    	CORT_Flag_Set(&i2c_flag, eI2CEvt_Error, CORT_OR);
	    }
	}
}

/*
** Open Device
*/
t_s8 dev_i2c_1_open(void)
{
	if (!i2c_1_init)
	{
		i2c_1_init = m_true;
		
		/* Unlock protected registers */		
		SYS_UnlockReg();

    	/* Enable IP clock */        
	    SYSCLK->APBCLK |= SYSCLK_APBCLK_I2C0_EN_Msk;

		/* set P3.4 and P3.5 for I2C0 SDA and SCL */
		SYS->P3_MFP |= SYS_MFP_P34_SDA0 | SYS_MFP_P35_SCL0;

		/* Lock protected registers */
		SYS_LockReg();
		
	    /* Reset I2C0 */
		SYS->IPRSTC2 |=  SYS_IPRSTC2_I2C0_RST_Msk;
		SYS->IPRSTC2 &= ~SYS_IPRSTC2_I2C0_RST_Msk;

		/* Enable I2C0 Controller */
		I2C0->I2CON |= I2C_I2CON_ENS1_Msk;
		
		/* I2C0 clock divider, I2C Bus Clock = PCLK / (4*120) = 100kHz */
//		I2C0->I2CLK = I2C_I2CLK_DIV4(120);
		I2C0->I2CLK = (SystemCoreClock/4/100000)-1;

		/* Set I2C0 4 Slave Addresses */            
		_I2C_SET_SLAVE_ADDRESS_0(I2C0, 0x15);   /* Slave Address : 0x15 */
		_I2C_SET_SLAVE_ADDRESS_1(I2C0, 0x35);   /* Slave Address : 0x35 */
		_I2C_SET_SLAVE_ADDRESS_2(I2C0, 0x55);   /* Slave Address : 0x55 */
		_I2C_SET_SLAVE_ADDRESS_3(I2C0, 0x75);   /* Slave Address : 0x75 */

		/* Enable I2C0 interrupt and set corresponding NVIC bit */
		I2C0->I2CON |= I2C_I2CON_EI_Msk;
		NVIC_EnableIRQ(I2C0_IRQn);
	}
	return CORT_OK;
}

/*
** IO Control
*/
t_s8 dev_i2c_1_ioctrl(t_u8 nParam, t_void* hParam, t_s32 nTimOutTick)
{
	t_i2c_set * i2c_set = (t_i2c_set*)hParam;
	t_i2c_opr * i2c_opr = (t_i2c_opr*)hParam;
	
	if (i2c_set && (nParam == eIOCtl_I2C_SET))
	{
		/* I2C0 clock divider, I2C Bus Clock = PCLK / (4*120) = 100kHz */
		I2C0->I2CLK = (SystemCoreClock/4/i2c_set->baud_rate)-1;

		return CORT_OK;
	}
	if (i2c_opr && (nParam == eIOCtl_I2C_OPR))
	{
		t_u32 val = 0;

		if (CORT_OK != CORT_Sem_Obtain(&i2c_sem, -1))
		{
			return m_false;
		}

		i2c_oper = *i2c_opr;
		i2c_reg_cnt = 0;
		i2c_read_cnt = 0;
		i2c_write_cnt = 0;

		/*Enable Interrupt & Enable I2C*/
		I2C0->I2CON |= I2C_I2CON_EI_Msk;

		/* I2C as master sends START signal */
		_I2C_START(I2C0);

		if (CORT_OK != CORT_Flag_Obtain(&i2c_flag, &val, eI2CEvt_MASK, CORT_SET_ANY | CORT_CONSUME, 200))
		{
			I2C0->I2CON &= ~I2C_I2CON_EI_Msk;
			CORT_Sem_Release(&i2c_sem);
			return m_false;
		}
		CORT_Sem_Release(&i2c_sem);
		if (val != eI2CEvt_OK)
			return CORT_ERROR;			
		return CORT_OK;
	}
	return CORT_ERR_INVALID_PARAM;
}

/*
** Standby Device
*/
t_s8 dev_i2c_1_standby(void)
{	
	return CORT_OK;
}

/*
** Resume Device for Standby
*/
t_s8 dev_i2c_1_resume(void)
{
	return CORT_OK;
}

/*
** Returns whether to allow the standby state
*/
t_bool dev_i2c_1_allowstandby(void)
{
	return m_true;
}

/*
** Initialize i2c device
*/
void dev_i2c_1_init(void)
{
	CORT_Sem_Create(&i2c_sem, 1);
	CORT_Flag_Create(&i2c_flag, 0);

	CORT_Interrupt_Register(34, dev_i2c_1_irq);
	
	i2c_1_dev.dev_type		= CORT_DAIT_I2C;
	i2c_1_dev.dev_idx		= 1;
	i2c_1_dev.open			= dev_i2c_1_open;;
	i2c_1_dev.ioctrl		= dev_i2c_1_ioctrl;
	i2c_1_dev.standby	 	= dev_i2c_1_standby;
	i2c_1_dev.resume	 	= dev_i2c_1_resume;
	i2c_1_dev.allow_standby = dev_i2c_1_allowstandby;
	
	CORT_DAI_RegDev(&i2c_1_dev);
}

